@@ -11,9 +11,14 @@ from viewmask.utils import (
     centers_to_image
 )
 from os.path import splitext
+from os import environ
+from functools import partial
 try:
     import napari
-except ImportError:
+    from napari.qt.threading import thread_worker
+except:
+    # don't just catch ImportError, since if *any* error
+    # is raised we should just proceed without napari.
     napari = None
 
 
@@ -23,12 +28,13 @@ NAPARI_NOT_INSTALLED_WARNING = "The package `napari` is not installed. " + \
 
 
 def validate_interactive_napari(ctx, param, value):
-    if value is None and napari is None:
-        raise click.BadParameter(
-            'The `interactive` flag cannot be passed '
-            'without `napari` in the environment. Please install the `napari` '
-            'package if you would like to use the `napari` interactive '
-            'viewer.')
+    if value is None:  # interactive mode
+        if napari is None:  # napari is not installed
+            raise click.BadParameter(
+                'The `interactive` flag cannot be passed '
+                'without `napari` in the environment. Please install the `napari` '
+                'package if you would like to use the `napari` interactive '
+                'viewer.')
     return value
 
 
@@ -39,6 +45,22 @@ def cli():
         from warnings import warn
         warn(NAPARI_NOT_INSTALLED_WARNING)
 
+    del_napari_async_env = False
+    previous_napari_async_env = None
+    if 'NAPARI_ASYNC' in environ:
+        previous_napari_async_env = environ['NAPARI_ASYNC']
+    else:
+        del_napari_async_env = True
+    # environ['NAPARI_ASYNC'] = "1"  # enable napari async
+
+
+@cli.resultcallback
+def shutdown():
+    if del_napari_async_env:
+        del environ['NAPARI_ASYNC']
+    else:
+        environ['NAPARI_ASYNC'] = previous_napari_async_env
+
 
 @cli.command(name='annotations')
 @click.argument(
@@ -74,10 +96,12 @@ def view_annotations(annotations, output):
                 shape_type='path',
                 edge_color=f"#{line_color}",
             )
-            viewer.add_points(
-                centers_of_contours(regions),
-                name='centers',
-            )
+            thread_worker(
+                centers_of_contours,
+                connect={
+                    "returned": partial(viewer.add_points, name='centers'),
+                },
+            )(regions)
     else:
         _, annotations_ext = splitext(annotations)
         if annotations_ext == '.npy':
@@ -127,9 +151,14 @@ def view_image(image):
     if napari is None:
         raise click.UsageError("The `image` command cannot be used without "
                                "the `napari` package installed.")
-    da_img = file_to_dask_array(image)
     with napari.gui_qt():
-        napari.view_image(da_img, name='image', multiscale=False)
+        viewer = napari.Viewer()
+        thread_worker(
+            file_to_dask_array,
+            connect={
+                "returned": partial(viewer.add_image, name='image', multiscale=False),
+            },
+        )(image)
 
 
 @cli.command(name='overlay')
@@ -173,10 +202,12 @@ def view_annotations(image, annotations, output):
                 shape_type='path',
                 edge_color=f"#{line_color}",
             )
-            viewer.add_points(
-                centers_of_contours(regions),
-                name='centers',
-            )
+            thread_worker(
+                centers_of_contours,
+                connect={
+                    "returned": partial(viewer.add_points, name='centers'),
+                },
+            )(regions)
     else:
         rendered_annotations = da_img
 
